% SP.signal Class
%
%   Defines a signal of the SP package, where an SP.signal object describes  
%   and contains typical signal with its properties such as the samplerate,  
%   the channel number and the signal length in seconds as well as 
%   samples. If a string to a file is provided it also saves the path, 
%   filename, type and extension.
%   Supported filetypes are .wav, .srt and .mat. It is also possible to
%   generate standard signals like silence or sinus oscillations.
%
% SP.signal Creation
%
%   mySignal = SP.signal
%
%   mySignal = SP.signal('/path/to/file.wav')
%   mySignal = SP.signal('/path/to/file.wav', channels)
%   mySignal = SP.signal('/path/to/file.srt', fs)
%   mySignal = SP.signal(waveform, fs, 'SourceName')
%   mySignal = SP.signal('silence', lengthInSeconds, fs, channels)
%   mySignal = SP.signal('sinus', lengthInSeconds, frequency, fs, channels)
%   mySignal = SP.signal('WhiteNoise', lengthInSeconds, fs, channels)
%   mySignal = SP.signal('BandpassNoise', lengthInSeconds, fs, channels, f_l, f_u)
%
% Basic Examples using SP.signal Class
%
%   % First create a sinusoidal, a bandpass filtered noise and a whitenoise
%   % signal
%   mySignal_sin        = SP.signal('sinus', 10, 440, 16000, 1)
%   mySignal_bandNoise  = SP.signal('BandpassNoise', 10, 16000, 1, 30, 300)
%   mySignal_whiteNoise = SP.signal('WhiteNoise', 8, 16000, 1)
%   
%   % Now calculate the sum, the difference and two scaled versions
%   mySum           = mySignal_sin + mySignal_bandNoise
%   mySubtraction   = mySignal_sin - mySignal_bandNoise
%   myScaled_1      = mySignal_sin .* 10
%   myScaled_2      = mySignal_whiteNoise ./ 10
%
%   % Adjust the longer signal to the length of the shorter one and
%   % calculate the sum
%   SP.truncate(myScaled_1, myScaled_2)
%   myScaledSum = myScaled_1 + myScaled_2
%
%   % Plot the spectogram and open a player to hear the sum
%   myScaledSum.spectogram
%   myScaledSum.play
%
%--------------------------------------------------------------------------
% Copyright (c) 2014, Florian Heese
% Institute of Communication Systems and Data Processing
% RWTH Aachen University, Germany
% Contact information: heese@ind.rwth-aachen.de
%
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
%
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in
%      the documentation and/or other materials provided with the 
%      distribution
%    * Neither the name of the RWTH Aachen University nor the names
%      of its contributors may be used to endorse or promote products 
%      derived from this software without specific prior written 
%      permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
% PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
% OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%--------------------------------------------------------------------------
classdef signal < handle

    properties
      metadata;
      analysisCallback = [];
   end
   
   properties (SetAccess = private)
      sourceName
      sourcePath
      sourceFilename
      sourceFilenameExtension
      sourceType
      fs
      channels
      signalLength
      signalSamples
   end
   
   properties (Access = private)
      signalDouble
      signalShort
   end
   
   methods
      function this = signal(varargin)
         
         my_nargin = nargin;
         
         % load Signal via file dialog if no arguments are given
         if nargin == 0
            global SP_signal_last_load_path
            
            myCurDir = cd;
            
            if isempty(SP_signal_last_load_path) || isnumeric(SP_signal_last_load_path) || ~exist(SP_signal_last_load_path, 'dir')
               SP_signal_last_load_path = pwd;
            end
            
            cd(SP_signal_last_load_path);
            
            [filename, pathname] = uigetfile(...
                {'*.wav;*.srt;*.mat','Audio Files (*.wav,*.srt,*.mat)';
                '*.wav',  'Waveform Audio File (*.wav)'; ...
                '*.srt','Short (*.srt)'; ...
                '*.mat','MAT-files (*.mat)'; ...
                '*.*',  'All Files (*.*)'}, ...
                'Select Audio File');

            SP_signal_last_load_path = pathname;
            
            cd(myCurDir);
            
            if isequal(filename, 0)
               error('User selected Cancel');
            end
            
            varargin{1} = fullfile(pathname, filename);
            
            [~, ~, EXT] = fileparts(varargin{1});
            
            if strcmpi(EXT, '.srt')
               varargin{2} = str2double(char(inputdlg(['Enter Frequency Samplerate of ' this.sourceName])));
               my_nargin = 2;
               fprintf('SP.signal(''%s'', %i)\n', varargin{1}, varargin{2})
            else
               my_nargin = 1;
               fprintf('SP.signal(''%s'')\n', varargin{1})
            end
         end
         
         if ischar(varargin{1})
            % check if arg1 is a file
            if exist(varargin{1}, 'file')
                this.load_from_file(varargin{:})
            else % arg1 (type character) is not a file
                switch varargin{1}
                    case 'silence'
                        this.sourceType    = 'Silence';
                        this.fs            = varargin{3};
                        this.channels      = varargin{4};
                        this.signalDouble  = zeros(this.fs .*varargin{2}, this.channels);
                        this.signalSamples = size(this.signalDouble, 1);
                        this.sourceName    = sprintf('%1.2f s of silence', varargin{2});
                        
                    case 'sinus'
                        this.sourceType    = 'Sinus';
                        this.fs            = varargin{4};
                        this.channels      = varargin{5};
                        this.signalDouble  = repmat(sin(linspace(0, varargin{2}, this.fs .*varargin{2}) .* 2 * pi * varargin{3} )', 1, varargin{5});
                        this.signalSamples = size(this.signalDouble, 1);
                        this.sourceName    = sprintf('%1.2fs of sin(2*pi*%1.2f Hz)', varargin{2}, varargin{3});

                    case 'WhiteNoise'
                        this.sourceType    = 'wgn';
                        this.fs            = varargin{3};
                        this.channels      = varargin{4};
                        tmp                = wgn(this.fs .*varargin{2}, this.channels, 0);
                        this.signalDouble  = tmp ./ max(abs(tmp(:)));
                        this.signalSamples = size(this.signalDouble, 1);
                        this.sourceName    = sprintf('%1.2f s of WGN', varargin{2});

                    otherwise
                        error('Could not interpret argument: ''%s''', varargin{1})
                end
            end

         elseif isnumeric(varargin{1})
            if my_nargin == 1,
               error('Sampling Frequency is missing!');
            end
            
            this.fs         = varargin{2};
            this.sourceType = 'from MATLAB variable';
            this.sourceName = inputname(1);
            
            if size(varargin{1},1) > size(varargin{1}, 2)
               data =  varargin{1};
            else
               data =  varargin{1}';
            end

            this.channels = size(data, 2);
            
            if max(abs(varargin{1})) > 2.0;
               this.signalShort   = data;
            else
               this.signalDouble  = data;
            end
            this.signalSamples = length(varargin{1});
            
            if nargin>2
               this.sourceName = varargin{3};
            end
         end
         
         if isempty(this.metadata)
            this.metadata = SP.metadata;
         end
         
         this.signalLength = this.signalSamples ./ this.fs;
         
      end
      
      function load_from_file(this, varargin)
            my_nargin = numel(varargin);
            
            if my_nargin > 0
                [PATHSTR, NAME, EXT] = fileparts(varargin{1});

                this.sourceName              = NAME;
                this.sourcePath              = PATHSTR;
                this.sourceFilenameExtension = EXT;
                this.sourceFilename          = varargin{1};
            end

            % try to load file depending on file extension
            switch lower(this.sourceFilenameExtension)
                case '.wav'
                  this.sourceType = 'Waveform Audio File Format (WAVE)';

                  [tmp_wav, this.fs] = wavread(this.sourceFilename, 'double');
                  if isempty(this.channels)
                      if my_nargin == 1
                         this.signalDouble = tmp_wav;
                      else
                         this.signalDouble = tmp_wav(:, varargin{2});
                      end
                      this.signalSamples = size(this.signalDouble, 1);
                      this.channels = size(this.signalDouble, 2);
                  else
                      this.signalDouble = tmp_wav(:, 1:this.channels);
                  end

               case '.mat' % detect .mat format
                  load(this.sourceFilename);

                  % from CoVR Recorings
                  if ( exist('metapar', 'var') && exist('simpar', 'var') )

                    if isequal( metapar.type, 'recording')
                      % multichannel audio recording
                      this.sourceType = 'MAT File Recording';
                      if my_nargin == 1
                        this.signalDouble = recording;
                      else
                        this.signalDouble = recording(varargin{2}, :);
                      end
                      this.fs = simpar.fs;
                      this.channels = size(this.signalDouble, 2);

                      this.metadata = SP.metadata;

                      my_fields = fieldnames(metapar);
                      for idx = 1:numel(my_fields);
                        this.metadata.addMetaData(my_fields{idx}, metapar.(my_fields{idx}));
                      end

                    elseif isequal(metapar.type, 'impulse_response')
                      % multichannel Impulse response

                      this.sourceType = 'MAT File Impulse Response';
                      if my_nargin == 1
                                this.signalDouble = impulse_response;
                      else
                                this.signalDouble = impulse_response(varargin{2}, :);
                      end
                      this.fs = simpar.fs;
                      this.channels = size(this.signalDouble, 2);

                      this.metadata = SP.metadata;

                      my_fields = fieldnames(metapar);
                      for idx = 1:numel(my_fields);
                          this.metadata.addMetaData(my_fields{idx}, metapar.(my_fields{idx}));
                      end                        

                    end

                    this.signalSamples = size(this.signalDouble, 1);
                  end

                  % SP.signal native format
                  if ( exist('sp_signal', 'var') && isstruct(sp_signal) )
                      % SP.Signal
                        this.signalDouble =            sp_signal.signalDouble;
                        this.fs =                      sp_signal.fs;
                        this.channels =                sp_signal.channels;
                        this.signalLength =            sp_signal.signalLength;
                        this.signalSamples =           sp_signal.signalSamples;
                        this.sourceName =              sp_signal.sourceName;
                        this.sourcePath =              sp_signal.sourcePath;
                        this.sourceFilename =          sp_signal.sourceFilename;
                        this.sourceFilenameExtension = sp_signal.sourceFilenameExtension;
                        this.sourceType =              sp_signal.sourceType;
                        this.metadata =                sp_signal.metadata;
                        this.analysisCallback =        sp_signal.analysisCallback;
                  end

                otherwise
                     error('Unknown file extension!')

            end
          
      end

      function copy = copy(this)
         copy = SP.signal(this.to_double, this.fs);
         copy.sourceName =                     this.sourceName;
         copy.sourcePath =                     this.sourcePath;
         copy.sourceFilename =                 this.sourceFilename;
         copy.sourceFilenameExtension =        this.sourceFilenameExtension;
         copy.sourceType =                     this.sourceType;
         copy.metadata =                       this.metadata.copy;
         copy.analysisCallback =               this.analysisCallback;
      end
   end
   
end

