% adjustSNR(signal, noise, SNR_db) returns a noisy SP.signal, which is 
%   mixed according to the  parameter SNR_dB. It consists of the clean 
%   signal plus a scaled version of the noise signal: 
%
%     noisy = signal + factor * noise,
%
%   with factor calculated according to:
%   
%     factor = sqrt(power(signal) / ( 10 ^ (SNR_dB / 10) * power(noise)
%
%   Note: If necessary, all signals (signal, noise, noisy) are normalized 
%         by a common factor with subject to max(abs(noisy)) < 1. In that
%         case the returned versions of signal and noise differ from their
%         input version!
%
%--------------------------------------------------------------------------
% Copyright (c) 2014, Florian Heese
% Institute of Communication Systems and Data Processing
% RWTH Aachen University, Germany
% Contact information: heese@ind.rwth-aachen.de
%
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
%
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in
%      the documentation and/or other materials provided with the 
%      distribution
%    * Neither the name of the RWTH Aachen University nor the names
%      of its contributors may be used to endorse or promote products 
%      derived from this software without specific prior written 
%      permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
% PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
% OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%--------------------------------------------------------------------------
function [ noisy, signal, noise, fac ] = adjustSNR( signal, noise, SNR_dB, channel_signal, channel_noise)

  % set channel for signal power
  if ~exist('channel_signal', 'var')
    channel_signal = 1:signal.channels;
  end

  % set channel for noise power
  if ~exist('channel_noise', 'var')
    channel_noise = 1:noise.channels;
  end
    
  if ~exist('fac','var')
    % compute signal and noise power
    P_s = mean(mean(signal.signalPower(:, channel_signal) ));
    P_n = mean(mean(noise.signalPower(:, channel_noise) ));
    
    % calculate scaling factor
    fac = sqrt( P_s / ( 10 ^ ( SNR_dB / 10 ) * P_n ) );
  end
  
  % Override Detection
  n_tmp = noise.to_double;
  s_tmp = signal.to_double;
  max_amplitude = max(max(abs(s_tmp + n_tmp .* fac)));
  
  if  max_amplitude > 1
      override_norm = 0.995 / max_amplitude;
      warning('Scaling signal and noise to prevent overriding (factor=%f)!', override_norm);
      
      
      signal = signal.normalize(override_norm.*max(max(abs(s_tmp))));
      signal.metadata.addMetaData('OverrideNormalization', override_norm);
      
      noise = noise.normalize(override_norm.*max(max(abs(n_tmp))));
      noise.metadata.addMetaData('OverrideNormalization', override_norm);
  end
  
  % adjust noise signal and compute noisy signal
  noise = noise .* fac;
  noisy = signal + noise;
  
  % add metadata
  noisy.metadata.addMetaData('SNR', SNR_dB);
  noisy.metadata.addMetaData('noiseScaleFactor', fac);
  
end