%loadImpulseResponse(db_path, mic_spacing, angle, distance, reverberation, truncate)
%   Loads an impulse response from db_path folder according to the
%   parameters microphone spacing, angle, distance and reverberation
%   time and returns the impulse response as SP.signal. The parameter
%   truncate is optional and allows to shorten the impulse response 
%   (unit is seconds).
%--------------------------------------------------------------------------
% Copyright (c) 2014, Florian Heese
% Institute of Communication Systems and Data Processing
% RWTH Aachen University, Germany
% Contact information: heese@ind.rwth-aachen.de
%
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
%
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in
%      the documentation and/or other materials provided with the 
%      distribution
%    * Neither the name of the RWTH Aachen University nor the names
%      of its contributors may be used to endorse or promote products 
%      derived from this software without specific prior written 
%      permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
% PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
% OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
% EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
% PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
% PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%--------------------------------------------------------------------------
function h = loadImpulseResponse(db_path, mic_spacing, angle, distance, reverberation, truncate)
    
    % autocorrect reverberation information
    if reverberation > 1
        reverberation = sprintf('%.3fs', reverberation/1000);
    else
        reverberation = sprintf('%.3fs', reverberation);
    end
    
    % create file list
    files = dir(db_path);
    % filter only files
    files = {files([files.isdir] == 0).name};
    % grep all files with specific reverberation (based on filename)
    my_files = grep(files, reverberation);
    % grep all files with specific microphone spacing (based on filename)
    my_files = grep(my_files, mic_spacing);
    % grep all files with specific distance (based on filename)
    my_files = grep(my_files, sprintf('%im', distance));
    % grep all files with specific angle (based on filename)
    angle = sprintf('%.3i.mat', angle);
    my_files = grep(my_files, angle);
    
    % check if only one file is left
    assert(numel(my_files) == 1);
    
    % load impulse response
    filename = fullfile(db_path, my_files{1});
    h = SP.signal(filename);
    
    if exist('truncate', 'var')
        h.cut(h.fs .* truncate); 
    end
    
    function array = grep(array, reg_exp)
        array = array(~cellfun(@isempty, regexpi(array, reg_exp)));
    end
end